unit Distribuzione;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ASIConst, PianoCartesiano;

type
  TDistribuzione = class(TComponent)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FMin, FMax, FSize: Extended;
    A, B: Extended;
    FIntCount: Integer;
    FTot, FNum: Integer;

    Valori: PExtendedArray0;

    procedure AddValue(Value: Extended);
    procedure SetMin(Value: Extended);
    procedure SetMax(Value: Extended);
    procedure SetIntCount(Value: Integer);
    function GetMedia: Extended;
    function GetVarianza: Extended;
    function GetFunzione: TFunzione;
    procedure SetAll;
    function Eval(X: Extended): Extended;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Valore: Extended write AddValue;
    property Min: Extended read FMin write SetMin;
    property Max: Extended read FMax write SetMax;
    property IntCount: Integer read FIntcount write SetIntCount;
    property Media: Extended read GetMedia;
    property Varianza: Extended read GetVarianza;
    property Funzione: TFunzione read GetFunzione;
    property Num: Integer read FNum;
    procedure Clear;
  published
    { Published declarations }
  end;

procedure Register;

implementation

type
  TDistFunz = class(TFunzione)
    Dist: TDistribuzione;
    constructor Create(ADist: TDistribuzione);
    function Valuta(X: Extended): Extended; override;
  end;

constructor TDistFunz.Create(ADist: TDistribuzione);
begin
  inherited Create;
  Dist := ADist;
end;

function TDistFunz.Valuta(X: Extended): Extended;
begin
  Result := Dist.Eval(X);
end;

constructor TDistribuzione.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMin := -10;
  FMax := +10;
  FIntCount := 1000;
  GetMem(Valori, FIntCount * SizeOf(Extended));
  SetAll;
end;

destructor TDistribuzione.Destroy;
begin
  FreeMem(Valori);
  inherited Destroy;
end;

procedure TDistribuzione.SetAll;
begin
  FillChar(Valori^, FIntCount * SizeOf(Extended), 0);
  FSize := (FMax - FMin) / FIntCount;
  FTot := 0;
end;

procedure TDistribuzione.SetMin(Value: Extended);
begin
  if FMin <> Value then begin
    FMin := Value;
    SetAll;
  end;
end;

procedure TDistribuzione.SetMax(Value: Extended);
begin
  if FMax <> Value then begin
    FMax := Value;
    SetAll;
  end;
end;

procedure TDistribuzione.SetIntCount(Value: Integer);
begin
  if FIntCount <> Value then begin
    FIntCount := Value;
    FreeMem(Valori);
    GetMem(Valori, FIntCount * SizeOf(Extended));
    SetAll;
  end;
end;

procedure TDistribuzione.AddValue(Value: Extended);
var
  I: Integer;
begin
  if (Value >= FMin) and (Value < FMax) then begin
    I := Trunc((Value - FMin) / FSize);
    Valori[I] := Valori[I] + 1;
    Inc(FTot);
  end;
  A := A + Value;
  B := B + Value * Value;
  Inc(FNum);
end;

function TDistribuzione.Eval(X: Extended): Extended;
var
  I: Integer;
begin
  Result := 0;
  if (X >= FMin) and (X < FMax) then begin
    I := Trunc((X - FMin) / FSize);
    if FTot <> 0 then Result := (Valori[I] / FTot) / FSize;
  end;
end;

function TDistribuzione.GetMedia: Extended;
begin
  if FNum <> 0 then
    Result := A / FNum
  else
    Result := 0;
end;

function TDistribuzione.GetVarianza: Extended;
begin
  if FNum > 1 then
    Result := (B - A * A / FNum) / (FNum - 1)
  else
    Result := 0;
end;

procedure TDistribuzione.Clear;
begin
  FTot := 0;
  FNum := 0;
  A := 0;
  B := 0;
  FillChar(Valori^, FIntCount * SizeOf(Extended), 0);
end;

function TDistribuzione.GetFunzione: TFunzione;
begin
  Result := TDistFunz.Create(Self);
end;

procedure Register;
begin
  RegisterComponents('AndSoft Inc.', [TDistribuzione]);
end;

end.
