unit NRC;

interface

uses ASIConst;

type
  PRowArray = ^TRowArray;
  TRowArray = array[1..10000] of PExtendedArray1;

type
  TMatrix = class(TObject)
  private
    FDim: Integer;
    FData: PRowArray;
    VV: PExtendedArray1;
    procedure SetData(Row: Integer; Col: Integer; Value: Extended);
    function GetData(Row: Integer; Col: Integer): Extended;
  public 
    constructor Create(ADim: Integer);
    destructor Destroy; override;
    property Dim: Integer read FDim;
    property Data[Row: Integer; Col: Integer]: Extended read GetData write SetData;
    procedure LUDCMP(Indx: PIntegerArray1; var D: Extended);
    procedure LUBKSB(Indx: PIntegerArray1; B: PExtendedarray1);
  end;
  
implementation

uses SysUtils;

const
  TINY = 1.0e-20;

constructor TMatrix.Create(ADim: Integer);
var
  I: Integer;
begin
  inherited Create;
  FDim := ADim;
  GetMem(FData, SizeOf(PExtendedArray0) * FDim);
  for I := 1 to FDim do begin
    GetMem(FData[I], SizeOf(Extended) * FDim);
    FillChar(FData[I]^, SizeOf(Extended) * FDim, 0);
  end;
  GetMem(VV, SizeOf(Extended) * FDim);
end;

destructor TMatrix.Destroy;
var
  I: Integer;
begin
  for I := 1 to FDim do begin
    FreeMem(FData[I]);
  end;
  FreeMem(FData);
  FreeMem(VV);
  inherited;
end;

function TMatrix.GetData(Row: Integer; Col: Integer): Extended;
begin
  Result := FData[Row][Col];
end;

procedure TMatrix.SetData(Row: Integer; Col: Integer; Value: Extended);
begin
  FData[Row][Col] := Value;
end;

procedure TMatrix.LUDCMP(Indx: PIntegerArray1; var D: Extended);
var
  I, IMax, J, K: Integer;
  Big, Dum, Sum, Temp: Extended;
begin
  D := 1;
  for I := 1 to FDim do begin
    Big := 0;
    for J := 1 to FDim do begin
      Temp := Abs(FData[I][J]);
      if Temp > Big then Big := Temp;
    end;
    if Big = 0 then raise Exception.Create('Singular matrix!');
    VV[I] := 1 / Big;
  end;
  for J := 1 to FDim do begin
    for I := 1 to J - 1 do begin
      Sum := FData[I][J];
      for K := 1 to I - 1 do
        Sum := Sum - FData[I][K] * FData[K][J];
      FData[I][J] := Sum;
    end;  
    Big := 0;
    for I := J to FDim do begin
      Sum := FData[I][J];
      for K := 1 to J - 1 do
        Sum := Sum - FData[I][K] * FData[K][J];
      FData[I][J] := Sum;
      Dum := VV[I] * Abs(Sum);
      if Dum >= Big then begin
      	  Big := Dum;
      	  IMax := I;
      end;      
    end;
    if J <> IMax then begin
      for K := 1 to FDim do begin
      	Dum := FData[IMax][K];
      	FData[IMax][K] := FData[J][K];
      	FData[J][K] := Dum;
      end;
      D := -D;
      VV[IMax] := VV[J];
    end;
    Indx[J] := IMax;
    if FData[J][J] = 0 then
      FData[J][J] := TINY;
    if J <> FDim then begin
      Dum := 1 / FData[J][J];
      for I := J + 1 to FDim do
        FData[I][J] := FData[I][J] * Dum;
    end;
  end;
end;

procedure TMatrix.LUBKSB(Indx: PIntegerArray1; B: PExtendedarray1);
var
  I, II, IP, J: Integer;
  Sum: Extended;
begin
  II := 0;
  for I := 1 to FDim do begin
    IP := Indx[I];
    Sum := B[IP];
    B[IP] := B[I];
    if II <> 0 then
      for J := II to I - 1 do
        Sum := Sum - FData[I][J] * B[J]
    else
      if Sum <> 0 then
        II := I;
    B[I] := Sum;
  end;
  for I := FDim downto 1 do begin
    Sum := B[I];
    for J := I + 1 to FDim do
      Sum := Sum - FData[I][J] * B[J];
    B[I] := Sum / FData[I][I];
  end;
end;

end.