unit Sistema;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RPNCompiler, ASIConst, NRC;

type
  PFormulaArray = ^TFormulaArray;
  TFormulaArray = array[1..65536] of TRPNCompiler;

  PNotifyEvent = ^TNotifyEvent;

  TDDEPacket = record
    DDEData: Pointer;
    DDESize: Integer;
  end;

type
  { componenti X da 1..
    componente B da 1..
    intervalli da 0.. }

  TSistema = class(TComponent)
  private
    { Private declarations }
    FOnRecalculate: TList;
    FOnFineRicalcolo: TNotifyEvent;
    FRicalcolaThread: TThread;

    function GetDerive(Index: Integer): TRPNCompiler;
    function GetOrigini(Index: Integer): Extended;
    procedure SetOrigini(Index: Integer; Value: Extended);
    function GetComponente(Index: Integer): TFunzione;
    function GetInterpolationPosition(X, N: Integer): Integer;
    function Eval(X: Integer; T: Extended): Extended;
    procedure NotifyRecalculate;
  protected
    { Protected declarations }
    FInizio, FFine, FPasso: Extended;
    FDimX: Integer;

    Mutex: TMultiReadExclusiveWriteSynchronizer;

    FIntervalli: Integer;

    FInterpolazione: PExtendedArray1;
    FInterpolazioneSize: Integer;

    FFormule: PFormulaArray;
    FFormuleNum: Integer;

    FOrigini: PExtendedArray1;
    FOriginiSize: Integer;

    procedure CheckX(X: Integer);
  public
    { Public declarations }
    property Derive[Index: Integer]: TRPNCompiler read GetDerive;
    property Origini[Index: Integer]: Extended read GetOrigini write SetOrigini;
    property Componente[Index: Integer]: TFunzione read GetComponente;
    property DimX: Integer read FDimX;
    property Inizio: Extended read FInizio;
    property Fine: Extended read FFine;
    property Passo: Extended read FPasso;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function AddOnRecalculate(NotifyEvent: TNotifyEvent): PNotifyEvent;
    procedure RemoveOnRecalculate(NotifyEvent: PNotifyEvent);
    procedure Ricalcola; virtual; abstract;
    procedure RicalcolaThread(N: Integer);
    procedure InterrompiRicalcola;
    procedure SetParametri(Inizio, Fine, Passo: Extended); virtual;
    function FastTableFormat(AInizio, AFine, APasso: Extended; ACount: Integer; AComponenti: PIntegerArray1): TDDEPacket;
  published
    { Published declarations }
    property OnFineRicalcolo: TNotifyEvent read FOnFineRicalcolo write FOnFineRicalcolo;
  end;

type
  { componente B da 1.. }

  TSistemaS05 = class(TSistema)
  private
    { Private declarations }
    FDimB: Integer;
    function GetDiffusioni(IndexX, IndexB: Integer): TRPNCompiler;
    procedure CheckB(B: Integer);
  protected
    { Protected declarations }
  public
    { Public declarations }
    property Diffusioni[IndexX: Integer; IndexB: Integer]: TRPNCompiler read GetDiffusioni;

    constructor Create(AOwner: TComponent); override;
    procedure SetDimension(DimX, DimB: Integer);
    procedure Ricalcola; override;
  published
    { Published declarations }
  end;

type
  TSistemaDRK = class(TSistema)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure SetDimension(DimX: Integer);
    procedure Ricalcola; override;
  published
    { Published declarations }
  end;

type
  TSistemaS10 = class(TSistema)
  private
    { Private declarations }
    function GetDiffusioni(IndexX: Integer): TRPNCompiler;
  protected
    { Protected declarations }
  public
    { Public declarations }
    property Diffusioni[IndexX: Integer]: TRPNCompiler read GetDiffusioni;
    procedure SetDimension(DimX: Integer);
    procedure Ricalcola; override;
  published
    { Published declarations }
  end;

type
  TSistemaW10 = class(TSistemaS05)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Ricalcola; override;
  published
    { Published declarations }
  end;

type
  TSistemaS15 = class(TSistema)
  private
    { Private declarations }
    function GetDiffusioni(IndexX: Integer): TRPNCompiler;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property Diffusioni[IndexX: Integer]: TRPNCompiler read GetDiffusioni;
    procedure Ricalcola; override;
  published
    { Published declarations }
  end;

type
  TSistemaDI1 = class(TSistema)
  private
    { Private declarations }
    FTemp: PExtendedArray1;
  protected
    { Protected declarations }
    procedure ZeroFunction(Arg: PExtendedArray0; Cons, ResOut: PExtendedArray1);
    procedure Jacobian(Arg: PExtendedArray0; Cons, ResIn: PExtendedArray1; Jac: TMatrix);
  public
    { Public declarations }
    procedure SetDimension(DimX: Integer);
    procedure Ricalcola; override;
  published
    { Published declarations }
  end;

const
  tdtTable  = $0010;
  tdtFloat  = $0001;
  tdtString = $0002;
  tdtBool   = $0003;
  tdtError  = $0004;
  tdtBlank  = $0005;
  tdtInt    = $0006;
  tdtSkip   = $0007;

type
  PFTF = ^TFTF;
  TFTF = packed record
    tdt: Word;
    cb: Word;
    case Word of
      tdtTable: (Rows, Cols: Word);
      tdtFloat: (dataB: array[0..65535] of Double);
  end;

implementation

var
  Old: Extended = 0;
  Presente: Boolean = False;

function GaussPolarMarsaglia: Extended;
var
  V1, V2: Extended;
  W: Extended;
begin
  if Presente then
    Result := Old
  else begin
    repeat
      V1 := 2.0 * Random - 1.0;
      V2 := 2.0 * Random - 1.0;
      W := V1 * V1 + V2 * V2;
    until (W <= 1.0) and (W > 0.0);
    W := Ln(W) / W;
    W := Sqrt(- W - W);
    Result := V1 * W;
    Old := V2 * W;
  end;
  Presente := not(Presente);
end;

type
  TComponente = class(TFunzione)
    Sistema: TSistema;
    N: Integer;
    constructor Create(ASistema: TSistema; AN: Integer);
    function Valuta(X: Extended): Extended; override;
  end;

constructor TComponente.Create(ASistema: TSistema; AN: Integer);
begin
  inherited Create;
  Sistema := ASistema;
  N := AN;
end;

function TComponente.Valuta(X: Extended): Extended;
begin
  Result := Sistema.Eval(N, X);
end;

type
  TRicalcolaThread = class(TThread)
  private
    { Private declarations }
    Sistema: TSistema;
    N: Integer;
    FError: string;
    procedure UpdateError;
  protected
    procedure Execute; override;
  public
    constructor Create(ASistema: TSistema; AN: Integer);
  end;

constructor TRicalcolaThread.Create(ASistema: TSistema; AN: Integer);
begin
  Sistema := ASistema;
  N := AN;
  inherited Create(False);
  FreeOnTerminate := True;
  OnTerminate := Sistema.OnFineRicalcolo;
end;

procedure TRicalcolaThread.UpdateError;
begin
  MessageDlg(FError, mtError, [mbOk], 0);
end;

procedure TRicalcolaThread.Execute;
var
  I: Integer;
  Buf: array[0..255] of Char;
begin
  try
    for I := 1 to N do begin
      if Terminated then Exit;
      Sistema.Ricalcola;
      Synchronize(Sistema.NotifyRecalculate);
    end;
  except
    ExceptionErrorMessage(ExceptObject, ExceptAddr, Buf, 255);
    FError := Buf;
    Synchronize(UpdateError);
  end;
end;

constructor TSistema.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInterpolazioneSize := 0;
  FOriginiSize := 0;
  FFormuleNum := 0;
  FIntervalli := 0;
  FOnRecalculate := TList.Create;
  FDimX := 0;
  Mutex := TMultiReadExclusiveWriteSynchronizer.Create;
  FRicalcolaThread := nil;
end;

destructor TSistema.Destroy;
var
  I: Integer;
begin
  if FFormuleNum <> 0 then begin
    for I := 1 to FFormuleNum do FFormule[I].Free;
    FreeMem(FFormule);
    FreeMem(FOrigini);
  end;
  if FInterpolazioneSize <> 0 then
    FreeMem(FInterpolazione);
  for I := 0 to FOnRecalculate.Count - 1 do
    Dispose(PNotifyEvent(FOnRecalculate.Items[I]));
  FOnRecalculate.Free;
  Mutex.Free;
  inherited Destroy;
end;

function TSistema.AddOnRecalculate(NotifyEvent: TNotifyEvent): PNotifyEvent;
var
  NE: PNotifyEvent;
begin
  New(NE);
  NE^ := NotifyEvent;
  FOnRecalculate.Add(NE);
  Result := NE
end;

procedure TSistema.RemoveOnRecalculate(NotifyEvent: PNotifyEvent);
begin
  FOnRecalculate.Remove(NotifyEvent);
  Dispose(NotifyEvent);
end;

procedure TSistema.NotifyRecalculate;
var
  I: Integer;
  P: PNotifyEvent;
begin
  for I := 0 to FOnRecalculate.Count - 1 do begin
    P := FOnRecalculate.Items[I];
    P^(Self);
  end;
end;

function TSistema.GetComponente(Index: Integer): TFunzione;
begin
  try
    Mutex.BeginRead;
    CheckX(Index);
    Result := TComponente.Create(Self, Index);
  finally
    Mutex.EndRead;
  end;
end;

procedure TSistema.CheckX(X: Integer);
begin
  if not(X in [1..FDimX]) then raise Exception.Create('Invalid dimension!');
end;

procedure TSistema.RicalcolaThread(N: Integer);
begin
  FRicalcolaThread := TRicalcolaThread.Create(Self, N);
end;

procedure TSistema.InterrompiRicalcola;
begin
  FRicalcolaThread.Terminate;
end;

procedure TSistema.SetParametri(Inizio, Fine, Passo: Extended);
begin
  Mutex.BeginWrite;
  FInizio := Inizio;
  FFine := Fine;
  FPasso := Passo;
  if FInterpolazioneSize <> 0 then
    FreeMem(FInterpolazione);
  FIntervalli := Trunc((FFine - FInizio) / FPasso) + 2;
  FInterpolazioneSize := FIntervalli * FDimX * SizeOf(Extended);
  GetMem(FInterpolazione, FInterpolazioneSize);
  FillChar(FInterpolazione^, FInterpolazioneSize, 0);
  Mutex.EndWrite;
end;

function TSistema.GetDerive(Index: Integer): TRPNCompiler;
begin
  try
    Mutex.BeginRead;
    CheckX(Index);
    Result := FFormule[Index];
  finally
    Mutex.EndRead;
  end;
end;

function TSistema.GetInterpolationPosition(X, N: Integer): Integer;
begin
  Result := X + N * FDimX;
end;

function TSistema.GetOrigini(Index: Integer): Extended;
begin
  try
    Mutex.BeginRead;
    CheckX(Index);
    Result := FOrigini[Index];
  finally
    Mutex.EndRead;
  end;
end;

procedure TSistema.SetOrigini(Index: Integer; Value: Extended);
begin
  try
    Mutex.BeginWrite;
    CheckX(Index);
    FOrigini[Index] := Value;
  finally
    Mutex.EndWrite;
  end;
end;

function TSistema.Eval(X: Integer; T: Extended): Extended;
var
  J: Integer;
begin
  try
    Mutex.BeginRead;
    CheckX(X);
    if (T >= FInizio) and (T <= FFine) then begin
      J := Trunc((T - FInizio) / FPasso);
      Result := FInterpolazione[GetInterpolationPosition(X, J)] +
        (FInterpolazione[GetInterpolationPosition(X, J + 1)] -
        FInterpolazione[GetInterpolationPosition(X, J)]) * (T - FInizio - FPasso * J) / FPasso;
    end else
      raise Exception.Create('Out of range!');
  finally
    Mutex.EndRead;
  end
end;

function TSistema.FastTableFormat(AInizio, AFine, APasso: Extended; ACount: Integer; AComponenti: PIntegerArray1): TDDEPacket;
var
  T: Extended;
  I, J, K, Size: Integer;
  FTF: PFTF;
begin
  I := Round((AFine - AInizio) / APasso);
  Size := SizeOf(Double) * ACount * (I + 1);

  Result.DDESize := 8 + 2 + 2 + Size;
  GetMem(Result.DDEData, Result.DDESize);

  FTF := PFTF(@(PByteArray1(Result.DDEData)^[1]));
  FTF.tdt := tdtTable;
  FTF.cb := 4;
  FTF.Rows := I + 1;
  FTF.Cols := ACount;

  FTF := PFTF(@(PByteArray1(Result.DDEData)^[9]));
  FTF.tdt := tdtFloat;
  FTF.cb := Size;
  for J := 0 to I do begin
    T := AInizio + J * APasso;
    for K := 1 to ACount do
      if AComponenti[K] = 0 then
        FTF.dataB[J * ACount + K - 1] := T
      else
        try
          FTF.dataB[J * ACount + K - 1] := Eval(AComponenti[K], T);
        except
          FTF.dataB[J * ACount + K - 1] := -1;
        end;
  end;
end;

constructor TSistemaS05.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDimB := 0;
end;

procedure TSistemaS05.SetDimension(DimX, DimB: Integer);
var
  I: Integer;
begin
  Mutex.BeginWrite;
  FDimX := DimX;
  FDimB := DimB;
  if FFormuleNum <> 0 then begin
    for I := 1 to FFormuleNum do FFormule[I].Free;
    FreeMem(FFormule);
    FreeMem(FOrigini);
  end;
  FFormuleNum := FDimX * (FDimB + 1);
  FOriginiSize := FDimX * SizeOf(Extended);
  GetMem(FFormule, FFormuleNum * SizeOf(TRPNCompiler));
  GetMem(FOrigini, FOriginiSize);
  for I := 1 to FFormuleNum do
    FFormule[I] := TRPNCompiler.Create(Self);
  SetParametri(FInizio, FFine, FPasso);
  Mutex.EndWrite;
end;

function TSistemaS05.GetDiffusioni(IndexX, IndexB: Integer): TRPNCompiler;
begin
  try
    Mutex.BeginRead;
    CheckX(IndexX);
    CheckB(IndexB);
    Result := FFormule[IndexB * FDimX + IndexX];
  finally
    Mutex.EndRead;
  end;
end;

procedure TSistemaS05.Ricalcola;
var
  Gaussiani: PExtendedArray1;
  Argomenti: PExtendedArray0;
  T: Extended;
  I, J, K: Integer;
begin
  Mutex.BeginWrite;
  GetMem(Gaussiani, SizeOf(Extended) * FDimB);
  GetMem(Argomenti, SizeOf(Extended) * (FDimX + 1));
  try
    for J := 1 to FDimX do
      FInterpolazione[GetInterpolationPosition(J, 0)] := FOrigini[J];
    for I := 1 to FIntervalli - 1 do begin
      T := FInizio + FPasso * (I - 1);
      Argomenti[0] := T;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)];
      for K := 1 to FDimB do
        Gaussiani[K] := GaussPolarMarsaglia * Sqrt(FPasso);
      for J := 1 to FDimX do begin
        FInterpolazione[GetInterpolationPosition(J, I)] :=
          FInterpolazione[GetInterpolationPosition(J, I - 1)] +
          GetDerive(J).Eval(Argomenti) * FPasso;
        for K := 1 to FDimB do
          FInterpolazione[GetInterpolationPosition(J, I)] :=
            FInterpolazione[GetInterpolationPosition(J, I)] +
            GetDiffusioni(J, K).Eval(Argomenti) * Gaussiani[K];
      end;
    end;
  finally
    FreeMem(Argomenti);
    FreeMem(Gaussiani);
    Mutex.EndWrite;
  end;
end;

procedure TSistemaS05.CheckB(B: Integer);
begin
  if not(B in [1..FDimB]) then raise Exception.Create('Invalid motion!');
end;

procedure TSistemaDRK.SetDimension(DimX: Integer);
var
  I: Integer;
begin
  Mutex.BeginWrite;
  FDimX := DimX;
  if FFormuleNum <> 0 then begin
    for I := 1 to FFormuleNum do
      FFormule[I].Free;
    FreeMem(FFormule);
    FreeMem(FOrigini);
  end;
  FFormuleNum := FDimX;
  FOriginiSize := FDimX * SizeOf(Extended);
  GetMem(FFormule, FFormuleNum * SizeOf(TRPNCompiler));
  GetMem(FOrigini, FOriginiSize);
  for I := 1 to FFormuleNum do
    FFormule[I] := TRPNCompiler.Create(Self);
  SetParametri(FInizio, FFine, FPasso);
  Mutex.EndWrite;
end;

procedure TSistemaDRK.Ricalcola;
var
  Argomenti: PExtendedArray0;
  T: Extended;
  I, J: Integer;
  K1, K2, K3, K4, K5, K6: PExtendedArray1;
begin
  Mutex.BeginWrite;
  GetMem(Argomenti, SizeOf(Extended) * (FDimX + 1));
  GetMem(K1, FDimX * SizeOf(Extended));
  GetMem(K2, FDimX * SizeOf(Extended));
  GetMem(K3, FDimX * SizeOf(Extended));
  GetMem(K4, FDimX * SizeOf(Extended));
  GetMem(K5, FDimX * SizeOf(Extended));
  GetMem(K6, FDimX * SizeOf(Extended));
  try
    for J := 1 to FDimX do
      FInterpolazione[GetInterpolationPosition(J, 0)] := FOrigini[J];

    for I := 1 to FIntervalli - 1 do begin
      T := FInizio + FPasso * (I - 1);

      (* K1 *)
      Argomenti[0] := T;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)];
      for J := 1 to FDimX do
        K1[J] := GetDerive(J).Eval(Argomenti) * FPasso;

      (* K2 *)
      Argomenti[0] := T + FPasso / 3;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)] + K1[J] / 3;
      for J := 1 to FDimX do
        K2[J] := GetDerive(J).Eval(Argomenti) * FPasso;

      (* K3 *)
      Argomenti[0] := T + FPasso * 2 / 5;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)] + (6 * K2[J] + 4 * K1[J]) / 25;
      for J := 1 to FDimX do
        K3[J] := GetDerive(J).Eval(Argomenti) * FPasso;

      (* K4 *)
      Argomenti[0] := T + FPasso;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)] + (15 * K3[J] - 12 * K2[J] + K1[J]) / 4;
      for J := 1 to FDimX do
        K4[J] := GetDerive(J).Eval(Argomenti) * FPasso;

      (* K5 *)
      Argomenti[0] := T + FPasso * 2 / 3;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)] + (8 * K4[J] - 50 * K3[J] + 90 * K2[J] + 6 * K1[J]) / 81;
      for J := 1 to FDimX do
        K5[J] := GetDerive(J).Eval(Argomenti) * FPasso;

      (* K6 *)
      Argomenti[0] := T + FPasso * 4 / 5;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)] + (8 * K4[J] + 10 * K3[J] + 36 * K2[J] + 6 * K1[J]) / 75;
      for J := 1 to FDimX do
        K6[J] := GetDerive(J).Eval(Argomenti) * FPasso;

      for J := 1 to FDimX do begin
        FInterpolazione[GetInterpolationPosition(J, I)] :=
          FInterpolazione[GetInterpolationPosition(J, I - 1)] +
          (23 * K1[J] + 125 * K3[J] - 81 * K5[J] + 125 * K6[J]) / 192;
      end;
    end;
  finally
    FreeMem(K1);
    FreeMem(K2);
    FreeMem(K3);
    FreeMem(K4);
    FreeMem(K5);
    FreeMem(K6);
    FreeMem(Argomenti);
    Mutex.EndWrite;
  end;
end;

procedure TSistemaS10.SetDimension(DimX: Integer);
var
  I: Integer;
begin
  Mutex.BeginWrite;
  FDimX := DimX;
  if FFormuleNum <> 0 then begin
    for I := 1 to FFormuleNum do
      FFormule[I].Free;
    FreeMem(FFormule);
    FreeMem(FOrigini);
  end;
  FFormuleNum := FDimX * 2;
  FOriginiSize := FDimX * SizeOf(Extended);
  GetMem(FFormule, FFormuleNum * SizeOf(TRPNCompiler));
  GetMem(FOrigini, FOriginiSize);
  for I := 1 to FFormuleNum do
    FFormule[I] := TRPNCompiler.Create(Self);
  SetParametri(FInizio, FFine, FPasso);
  Mutex.EndWrite;
end;

function TSistemaS10.GetDiffusioni(IndexX: Integer): TRPNCompiler;
begin
  CheckX(IndexX);
  Result := FFormule[FDimX + IndexX];
end;

procedure TSistemaS10.Ricalcola;
var
  Gaussiani: PExtendedArray1;
  Supporti1: PExtendedArray0;
  Argomenti, Supporti2: PExtendedArray0;
  T, D, G: Extended;
  I, J: Integer;
begin
  Mutex.BeginWrite;
  GetMem(Gaussiani, SizeOf(Extended) * FDimX);
  GetMem(Supporti1, SizeOf(Extended) * (FDimX + 1));
  GetMem(Supporti2, SizeOf(Extended) * (FDimX + 1));
  GetMem(Argomenti, SizeOf(Extended) * (FDimX + 1));
  try
    for J := 1 to FDimX do
      FInterpolazione[GetInterpolationPosition(J, 0)] := FOrigini[J];
    for I := 1 to FIntervalli - 1 do begin
      T := FInizio + FPasso * (I - 1);
      Argomenti[0] := T;
      for J := 1 to FDimX do begin
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)];
        Gaussiani[J] := GaussPolarMarsaglia * Sqrt(FPasso);
      end;
      Supporti1[0] := T;
      for J := 1 to FDimX do
        Supporti1[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)] +
          GetDerive(J).Eval(Argomenti) * FPasso;
      for J := 1 to FDimX do begin
        Move(Supporti1^, Supporti2^, SizeOf(Extended) * (FDimX + 1));
        D := GetDiffusioni(J).Eval(Argomenti);
        Supporti2[J] := Supporti2[J] + D * Sqrt(FPasso);
        G := Gaussiani[J];
        FInterpolazione[GetInterpolationPosition(J, I)] :=
          Supporti1[J] +
          D * G +
          (GetDiffusioni(J).Eval(Supporti2) - D) * (G * G - FPasso) / (2 * Sqrt(FPasso));
      end;
    end;
  finally
    FreeMem(Argomenti);
    FreeMem(Supporti2);
    FreeMem(Supporti1);
    FreeMem(Gaussiani);
    Mutex.EndWrite;
  end;
end;

procedure TSistemaW10.Ricalcola;
var
  Gaussiani: PExtendedArray1;
  Argomenti: PExtendedArray0;
  T: Extended;
  I, J, K: Integer;
begin
  Mutex.BeginWrite;
  GetMem(Gaussiani, SizeOf(Extended) * FDimB);
  GetMem(Argomenti, SizeOf(Extended) * (FDimX + 1));
  try
    for J := 1 to FDimX do
      FInterpolazione[GetInterpolationPosition(J, 0)] := FOrigini[J];
    for I := 1 to FIntervalli - 1 do begin
      T := FInizio + FPasso * (I - 1);
      Argomenti[0] := T;
      for J := 1 to FDimX do
        Argomenti[J] := FInterpolazione[GetInterpolationPosition(J, I - 1)];
      for K := 1 to FDimB do
        Gaussiani[K] := (Trunc(Random(2)) - 0.5) * 2 * Sqrt(FPasso);
      for J := 1 to FDimX do begin
        FInterpolazione[GetInterpolationPosition(J, I)] :=
          FInterpolazione[GetInterpolationPosition(J, I - 1)] +
          GetDerive(J).Eval(Argomenti) * FPasso;
        for K := 1 to FDimB do
          FInterpolazione[GetInterpolationPosition(J, I)] :=
            FInterpolazione[GetInterpolationPosition(J, I)] +
            GetDiffusioni(J, K).Eval(Argomenti) * Gaussiani[K];
      end;
    end;
  finally;
    FreeMem(Argomenti);
    FreeMem(Gaussiani);
    Mutex.EndWrite;
  end;
end;

constructor TSistemaS15.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDimX := 1;
  FFormuleNum := 2;
  FOriginiSize := SizeOf(Extended);
  GetMem(FFormule, FFormuleNum * SizeOf(TRPNCompiler));
  GetMem(FOrigini, FOriginiSize);
  FFormule[1] := TRPNCompiler.Create(Self);
  FFormule[2] := TRPNCompiler.Create(Self);
end;

function TSistemaS15.GetDiffusioni(IndexX: Integer): TRPNCompiler;
begin
  CheckX(IndexX);
  Result := FFormule[2];
end;

procedure TSistemaS15.Ricalcola;
var
  W, U1, U2: Extended;
  Y1, Y2, F1, F2: Extended;
  A, A1, A2, A3: Extended;
  B, B1, B2, B3, B4, B5: Extended;
  I01, I10, I11, I00, I111: Extended;
  Y, S, T: Extended;
  I: Integer;
  Dif, Der: TRPNCompiler;
begin
  Mutex.BeginWrite;
  FInterpolazione[GetInterpolationPosition(1, 0)] := FOrigini[1];
  S := Sqrt(FPasso);
  Der := GetDerive(1);
  Dif := GetDiffusioni(1);
  for I := 1 to FIntervalli - 1 do begin
    T := Inizio + FPasso * (I - 1);

    U1 := GaussPolarMarsaglia;
    U2 := GaussPolarMarsaglia;
    W := U1 * S;
    I10 := Exp(1.5 * Ln(FPasso)) * (U1 + U2 / Sqrt(3)) / 2;
    I00 := FPasso * FPasso / 2;
    I01 := W * FPasso - I10;
    I11 := (W * W - FPasso) / 2;
    I111 := (W * W / 3 - FPasso) * W / 2;

    Y := FInterpolazione[GetInterpolationPosition(1, I - 1)];

    A := Der.Eval([T, Y]);
    B := Dif.Eval([T, Y]);

    Y1 := Y + A * FPasso + B * S;
    Y2 := Y + A * FPasso - B * S;

    A1 := Der.Eval([T, Y1]);
    A2 := Der.Eval([T, Y2]);

    B1 := Dif.Eval([T, Y1]);
    B2 := Dif.Eval([T, Y2]);

    F1 := Y1 + B1 * S;
    F2 := Y1 - B1 * S;

    B3 := Dif.Eval([T, F1]);
    B4 := Dif.Eval([T, F2]);

    A3 := Der.Eval([T + FPasso, Y]);
    B5 := Dif.Eval([T + FPasso, Y]);

    FInterpolazione[GetInterpolationPosition(1, I)] := Y +
      A * FPasso +
      B * W +
      ((A1 - A2) * I10 + (B1 - B2) * I11) / (2 * S) +
      ((A3 - A) * I00 + (B5 - B) * I01) / FPasso +
      ((A1 - 2 * A + A2) * I00 + (B1 - 2 * B + B2) * I01) / (2 * FPasso) +
      (B3 - B4 - B1 + B2) * I111 / (2 * FPasso);
  end;
  Mutex.EndWrite;
end;

procedure TSistemaDI1.SetDimension(DimX: Integer);
var
  I: Integer;
begin
  Mutex.BeginWrite;
  FDimX := DimX;
  if FFormuleNum <> 0 then begin
    for I := 1 to FFormuleNum do
      FFormule[I].Free;
    FreeMem(FFormule);
    FreeMem(FOrigini);
  end;
  FFormuleNum := FDimX;
  FOriginiSize := FDimX * SizeOf(Extended);
  GetMem(FFormule, FFormuleNum * SizeOf(TRPNCompiler));
  GetMem(FOrigini, FOriginiSize);
  for I := 1 to FFormuleNum do
    FFormule[I] := TRPNCompiler.Create(Self);
  SetParametri(FInizio, FFine, FPasso);
  Mutex.EndWrite;
end;

procedure TSistemaDI1.Ricalcola;
const
  NTRIAL: Integer = 100;
  TOLX: Extended = 0.000001;
  TOLF: Extended = 0.000001;
var
  Argomenti: PExtendedArray0;
  Vecchi, ZF, P: PExtendedArray1;
  Indx: PIntegerArray1;
  D, T, ErrF, ErrX: Extended;
  I, J, K: Integer;
  FJac: TMatrix;
begin
  GetMem(Argomenti, SizeOf(Extended) * (FDimX + 1));
  GetMem(ZF, SizeOf(Extended) * FDimX);
  GetMem(Vecchi, SizeOf(Extended) * FDimX);
  GetMem(P, SizeOf(Extended) * FDimX);
  GetMem(Indx, SizeOf(Integer) * FDimX);
  GetMem(FTemp, SizeOf(Extended) * FDimX);
  FJac := TMatrix.Create(FDimX);
  Mutex.BeginWrite;
  try
    for J := 1 to FDimX do begin
      FInterpolazione[GetInterpolationPosition(J, 0)] := FOrigini[J];
      Argomenti[J] := FOrigini[J];
    end;
    for I := 1 to FIntervalli - 1 do begin
      T := FInizio + FPasso * (I - 1);
      Argomenti[0] := T;
      for J := 1 to FDimX do
        Vecchi[J] := Argomenti[J];	(* initial guess *)
      for K := 1 to NTRIAL do begin
        ErrF := 0;
        ZeroFunction(Argomenti, Vecchi, ZF);
        Jacobian(Argomenti, Vecchi, ZF, FJac);
        for J := 1 to FDimX do
          ErrF := ErrF + Abs(ZF[J]);
        if ErrF <= TOLF then BREAK; 
        for J := 1 to FDimX do
          P[J] := - ZF[J];
        FJac.LUDCMP(Indx, D);
        FJac.LUBKSB(Indx, P);
        ErrX := 0;
        for J := 1 to FDimX do begin
          ErrX := ErrX + Abs(P[J]);
          Argomenti[J] := Argomenti[J] + P[J];
        end;
        if ErrX <= TOLX then BREAK; 
      end;
      for J := 1 to FDimX do
        FInterpolazione[GetInterpolationPosition(J, I)] := Argomenti[J];
    end;
  finally
    Mutex.EndWrite;
    FJac.Free;
    FreeMem(Indx);
    FreeMem(P);
    FreeMem(ZF);
    FreeMem(Vecchi);
    FreeMem(Argomenti);
    FreeMem(FTemp);
  end;
end;

procedure TSistemaDI1.ZeroFunction(Arg: PExtendedArray0; Cons, ResOut: PExtendedArray1);
var
  I: Integer;
begin
  for I := 1 to FDimX do
    ResOut[I] := Arg[I] - Cons[I] - FPasso * GetDerive(I).Eval(Arg);
end;

procedure TSistemaDI1.Jacobian(Arg: PExtendedArray0; Cons, ResIn: PExtendedArray1; Jac: TMatrix);
const
  EPS = 1.0e-4;
var
  I, J: Integer;
  Temp, H: Extended;
begin
  for J := 1 to FDimX do begin
    Temp := Arg[J];
    H := EPS * Abs(Temp);
    if H = 0 then H := EPS;
    Arg[J] := Temp + H;
    H := Arg[J] - Temp;
    ZeroFunction(Arg, Cons, FTemp);
    Arg[J] := Temp;
    for I := 1 to FDimX do
      Jac.Data[I, J] := (FTemp[I] - ResIn[I]) / H;
  end;
end;

end.
