unit Orologio;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls;

type
  TOrologio = class(TGraphicControl)
  private
    { Private declarations }
    FOra: TDateTime;
    procedure SetOra(Value: TDateTime);
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property Ora: TDateTime write SetOra;
  published
    { Published declarations }
  end;

procedure Register;

implementation

uses Graphics, Math;

constructor TOrologio.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
end;

procedure TOrologio.SetOra(Value: TDateTime);
begin
  if Value <> FOra then begin
    FOra := Value;
    Repaint;
  end;
end;

procedure TOrologio.Paint;
var
  Bitmap: TBitmap;
  Rect: TRect;
  Hour, Min, Sec, MSec: Word;
  Angle: Double;
  Centro, Punto: TPoint;
begin
  Bitmap := TBitmap.Create;
  Rect := GetClientRect;
  Bitmap.Width := Rect.Right;
  Bitmap.Height := Rect.Bottom;

  Centro.X := Rect.Right div 2;
  Centro.Y := Rect.Bottom div 2;
  DecodeTime(FOra, Hour, Min, Sec, MSec);

  with Bitmap.Canvas do begin
    Brush.Color := Self.Color;
    FillRect(Rect);

    Pen.Width := 1;
    Pen.Color := clBlack;
    Angle := DegToRad(90 - Sec * 6.0);
    Punto.X := Trunc(Centro.X + Cos(Angle) * Rect.Right * 0.5);
    Punto.Y := Trunc(Centro.Y - Sin(Angle) * Rect.Bottom * 0.5);
    Arc(0, 0, Rect.Right, Rect.Bottom, Punto.X, Punto.Y, Centro.X, 0);

    Brush.Color := clRed;
    Pen.Color := clRed;
    Ellipse(Punto.X - 5, Punto.Y - 5, Punto.X + 5, Punto.Y + 5);

    Pen.Width := 2;
    Pen.Color := clYellow;
    Angle := DegToRad(90 - (Min + Sec / 60.0) * 6.0);
    Punto.X := Trunc(Centro.X + Cos(Angle) * Rect.Right * 0.5);
    Punto.Y := Trunc(Centro.Y - Sin(Angle) * Rect.Bottom * 0.5);
    MoveTo(Centro.X, Centro.Y);
    LineTo(Punto.X, Punto.Y);

    Pen.Color := clBlue;
    Angle := DegToRad(90 - (Hour + (Min + Sec / 60.0) / 60.0) * 30.0);
    Punto.X := Trunc(Centro.X + Cos(Angle) * Rect.Right * 0.3);
    Punto.Y := Trunc(Centro.Y - Sin(Angle) * Rect.Bottom * 0.3);
    MoveTo(Centro.X, Centro.Y);
    LineTo(Punto.X, Punto.Y);

  end;

  Canvas.Draw(0, 0, Bitmap);
  Bitmap.Free;
end;

procedure Register;
begin
  RegisterComponents('AndSoft Inc.', [TOrologio]);
end;

end.
