unit PianoCartesiano;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ASISpin, ASIConst, SyncObjs;

type
  TPCMode = (pcmZoom, pcmMove, pcmInvZoom, pcmNone);
  TCoordinate = array[0..3] of Extended;

type
  TPianoCartesiano = class(TGraphicControl)
  private
    { Private declarations }
    FCoordinate: TCoordinate;
    FFocusRect: TRect;
    FAuto: Boolean;
    FAggiorna: TNotifyEvent;
    FPCMode: TPCMode;
    FStart: TPoint;
    FDelta: TPoint;
    FStCoord: TCoordinate;
    FDCW, FDCH: Integer;
    FDRect: TRect;
    procedure SetPCMode(Value: TPCMode);
    function GetCoordinate(Index: Integer): Extended;
    procedure SetCoordinate(Index: Integer; Value: Extended);
    function CheckCoordinate(C1, C3: Extended): Boolean;
    procedure FMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  protected
    { Protected declarations }
    FPaintThread: TThread;
    FRestart: Boolean;
    procedure Paint; override;
    procedure RepaintAll;
    procedure StopAndWait;
    procedure PrepareDisegna;
    procedure Disegna;
    procedure DoneThread(Sender: TObject);
    procedure DisegnaVCL(Bitmap: TBitmap); virtual;
    procedure DisegnaThread(Th: TThread); virtual;
    function Delphi2RealX(X: Integer): Extended;
    function Delphi2RealY(Y: Integer): Extended;
    function Real2DelphiX(X: Extended): Integer;
    function Real2DelphiY(Y: Extended): Integer;
  public
    { Public declarations }
    property MinDXY[Indice: Integer]: Extended read GetCoordinate write SetCoordinate;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Align;
    property MinX: Extended index 0 read GetCoordinate write SetCoordinate;
    property DX: Extended index 1 read GetCoordinate write SetCoordinate;
    property MinY: Extended index 2 read GetCoordinate write SetCoordinate;
    property DY: Extended index 3 read GetCoordinate write SetCoordinate;
    property PCMode: TPCMode read FPCMode write SetPCMode default pcmNone;
    property OnAggiorna: TNotifyEvent read FAggiorna write FAggiorna;
  end;

  TPianoTraiettorie = class(TPianoCartesiano)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FFunzioni: TList;
    FMulti: TList;
    procedure DisegnaThread(Th: TThread); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddFunzione(Funzione: TFunzione);
    procedure RemoveFunzione(Funzione: TFunzione);
    procedure AddMulti(Funzione: TMultiFunzione);
    procedure RemoveMulti(Funzione: TMultiFunzione);
    procedure RemoveAll;
  published
    { Published declarations }
  end;

type
  TPianoIterazione = class(TPianoCartesiano)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FFunzione: TFunzione;
    FStart: Extended;
    FSkip: Integer;
    FIter: Integer;
    procedure DisegnaVCL(Bitmap: TBitmap); override;
    procedure DisegnaThread(Th: TThread); override;
    procedure SetFunzione(Value: TFunzione);
    procedure SetStart(Value: Extended);
    procedure SetIter(Value: Integer);
    procedure SetSkip(Value: Integer);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property Funzione: TFunzione read FFunzione write SetFunzione;
    property Start: Extended read FStart write SetStart;
    property Iter: Integer read FIter write SetIter;
    property Skip: Integer read FSkip write SetSkip;
  published
    { Published declarations }
  end;

type
  TPianoIterata = class(TPianoCartesiano)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FStart: Extended;
    FLambda: Extended;
    FFunzione: TFunzione;
    FMulti: TMultiFunzione;
    procedure DisegnaVCL(Bitmap: TBitmap); override;
    procedure DisegnaThread(Th: TThread); override;
    procedure SetStart(Value: Extended);
    procedure SetLambda(Value: Extended);
    procedure SetFunzione(Value: TFunzione);
    procedure SetMulti(Value: TMultiFunzione);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property Start: Extended read FStart write SetStart;
    property Lambda: Extended read FLambda write SetLambda;
    property Funzione: TFunzione read FFunzione write SetFunzione;
    property Multi: TMultiFunzione read FMulti write SetMulti;
  published
    { Published declarations }
  end;

type
  TPianoFasi = class(TPianoCartesiano)
  private
    { Private declarations }
    FX, FY: TFunzione;
    FInizio, FFine, FPasso: Extended;
    procedure SetX(Value: TFunzione);
    procedure SetY(Value: TFunzione);
  protected
    { Protected declarations }
    procedure DisegnaThread(Th: TThread); override;
  public
    { Public declarations }
    property X: TFunzione read FX write SetX;
    property Y: TFunzione read FY write SetY;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetParametri(Inizio, Fine, Passo: Extended);
  published
    { Published declarations }
  end;

type
  TSizePianoCartesiano = class(TCustomPanel)
  private
    { Private declarations }
    FPianoCartesiano: TPianoCartesiano;
    procedure SetPianoCartesiano(APianoCartesiano: TPianoCartesiano);
    procedure OnChangeMinDXY(Sender: TObject);
  protected
    { Protected declarations }
    FMinMaxXY: array[0..3] of TFloatSpinEdit;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Aggiorna(Sender: TObject);
  published
    { Published declarations }
    property PianoCartesiano: TPianoCartesiano read FPianoCartesiano write SetPianoCartesiano;
    property Align;
  end;

(*$R PianoCartesiano.dcr *)

procedure Register;

implementation

uses StdCtrls, Math;

type
  TRepaintThread = class(TThread)
  private
    { Private declarations }
    PC: TPianoCartesiano;
  protected
    procedure Execute; override;
  public
    constructor Create(APC: TPianoCartesiano);
    property Terminated;
  end;

constructor TRepaintThread.Create(APC: TPianoCartesiano);
begin
  inherited Create(True);
  PC := APC;
  PC.Disegna;
  FreeOnTerminate := True;
  OnTerminate := PC.DoneThread;
end;

procedure TRepaintThread.Execute;
begin
  if not(Terminated) then begin
    PC.DisegnaThread(Self);
  end;
end;

function RaddrizzaRect(R: TRect): TRect;
begin
  if R.Left <= R.Right then begin
    Result.Left := R.Left;
    Result.Right := R.Right;
  end else begin
    Result.Left := R.Right;
    Result.Right := R.Left;
  end;
  if R.Top <= R.Bottom then begin
    Result.Top := R.Top;
    Result.Bottom := R.Bottom;
  end else begin
    Result.Top := R.Bottom;
    Result.Bottom := R.Top;
  end;
end;

constructor TPianoCartesiano.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OnMouseDown := FMouseDown;
  OnMouseUp := FMouseUp;
  OnMouseMove := FMouseMove;
  ControlStyle := ControlStyle + [csOpaque];
  FAuto := False;
  FPCMode := pcmNone;
  FCoordinate[0] := 0;
  FCoordinate[1] := 10;
  FCoordinate[2] := -10;
  FCoordinate[3] := 20;
end;

destructor TPianoCartesiano.Destroy;
begin
  StopAndWait;
  inherited Destroy;
end;

procedure TPianoCartesiano.DoneThread(Sender: TObject);
begin
  FPaintThread := nil;
  if FRestart then Invalidate;
end;

procedure TPianoCartesiano.StopAndWait;
var
  FT: TThread;
begin
  if FPaintThread <> nil then begin
    FT := FPaintThread;
    FRestart := False;
    try
      FT.FreeOnTerminate := False;
      FT.Terminate;
      FT.WaitFor;
      FT.Free;
    except
      FT := nil;
    end;
  end;
end;

procedure TPianoCartesiano.RepaintAll;
begin
  PrepareDisegna;
  FRestart := False;
  FPaintThread := TRepaintThread.Create(Self);
  FPaintThread.Resume;
end;

procedure TPianoCartesiano.Paint;
begin
  if FPaintThread <> nil then begin
    FRestart := True;
    FPaintThread.Terminate;
  end else
    RepaintAll;
end;

procedure TPianoCartesiano.PrepareDisegna;
begin
  FDRect := Canvas.ClipRect;
  FDCH := ClientHeight;
  FDCW := ClientWidth;
end;

procedure TPianoCartesiano.Disegna;
var
  Bitmap: TBitmap;
begin
  Bitmap := TBitmap.Create;
  Bitmap.Width := FDRect.Right;
  Bitmap.Height := FDRect.Bottom;
  DisegnaVCL(Bitmap);
  Canvas.Draw(0, 0, Bitmap);
  Bitmap.Free;
end;

procedure TPianoCartesiano.DisegnaVCL(Bitmap: TBitmap);
begin
  with Bitmap.Canvas do begin
    Brush.Color := Self.Color;
    FillRect(FDRect);
    Pen.Color := clBlack;
    MoveTo(0, Real2DelphiY(0));
    LineTo(FDRect.Right, Real2DelphiY(0));
    MoveTo(Real2DelphiX(0), FDRect.Bottom);
    LineTo(Real2DelphiX(0), 0);
  end;
end;

procedure TPianoCartesiano.DisegnaThread(Th: TThread);
begin
end;

function TPianoCartesiano.GetCoordinate(Index: Integer): Extended;
begin
  Result := FCoordinate[Index];
end;

procedure TPianoCartesiano.SetCoordinate(Index: Integer; Value: Extended);
var
  OldValue: Extended;
begin
  if FCoordinate[Index] <> Value then begin
    StopAndWait;
    OldValue := FCoordinate[Index];
    FCoordinate[Index] := Value;
    try
      if not(CheckCoordinate(FCoordinate[1], FCoordinate[3])) then begin
        FCoordinate[Index] := OldValue;
        raise Exception.Create('Invalid coordinate!');
      end;
    finally
      Invalidate;
    end;
  end;
end;

function TPianoCartesiano.CheckCoordinate(C1, C3: Extended): Boolean;
begin
  Result := (C1 > 0) and (C3 > 0);
end;

(*
function TPianoCartesiano.Delphi2RealX(X, X1, X2: Integer): Extended;
begin
  Result := FCoordinate[1] * (X - X1) / (X2 - X1) + FCoordinate[0];
end;

function TPianoCartesiano.Delphi2RealY(Y, Y1, Y2: Integer): Extended;
begin
  Result := FCoordinate[2] + FCoordinate[3] * (Y2 - Y) / (Y2 - Y1);
end;

function TPianoCartesiano.Real2DelphiX(X, X1, X2: Extended): Integer;
begin
  Result := Round((X - FCoordinate[0]) * (X2 - X1) / FCoordinate[1] + X1);
end;

function TPianoCartesiano.Real2DelphiY(Y, Y1, Y2: Extended): Integer;
begin
  Result := Round(Y2 - (Y - FCoordinate[2]) * (Y2 - Y1) / FCoordinate[3]);
end;
*)

function TPianoCartesiano.Delphi2RealX(X: Integer): Extended;
begin
  Result := FCoordinate[1] * X / ClientWidth + FCoordinate[0];
end;

function TPianoCartesiano.Delphi2RealY(Y: Integer): Extended;
begin
  Result := FCoordinate[2] + FCoordinate[3] * (ClientHeight - Y) / ClientHeight;
end;

function TPianoCartesiano.Real2DelphiX(X: Extended): Integer;
begin
  Result := Round((X - FCoordinate[0]) * ClientWidth / FCoordinate[1]);
end;

function TPianoCartesiano.Real2DelphiY(Y: Extended): Integer;
begin
  Result := Round(ClientHeight - (Y - FCoordinate[2]) * ClientHeight / FCoordinate[3]);
end;

procedure TPianoCartesiano.FMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FPCMode = pcmNone then Exit;
  if Button = mbLeft then begin
    case FPCMode of
      pcmZoom, pcmInvZoom:
        begin
          FFocusRect.TopLeft.X := X;
          FFocusRect.TopLeft.Y := Y;
          FFocusRect.BottomRight := FFocusRect.TopLeft;
        end;
      pcmMove:
        begin
          FStart.X := X;
          FStart.Y := Y;
          FStCoord := FCoordinate;
        end;
    end;
    FAuto := True;
  end;
end;

procedure TPianoCartesiano.FMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  R: TRect;
  C: array[0..3] of Extended;
begin
  if FPCMode = pcmNone then Exit;
  if Button = mbLeft then begin
    FAuto := False;
    case FPCMode of
      pcmZoom:
        begin
          R := RaddrizzaRect(FFocusRect);
          Canvas.Lock;
          Canvas.DrawFocusRect(R);
          Canvas.UnLock;
          if (R.Right > R.Left) and (R.Bottom > R.Top) then begin
            C[0] := Delphi2RealX(R.Left);
            C[1] := Delphi2RealX(R.Right) - C[0];
            C[2] := Delphi2RealY(R.Bottom);
            C[3] := Delphi2RealY(R.Top) - C[2];
            StopAndWait;
            Move(C, FCoordinate, SizeOf(C));
            Invalidate;
          end;
        end;
      pcmInvZoom:
        begin
          R := RaddrizzaRect(FFocusRect);
          Canvas.Lock;
          Canvas.DrawFocusRect(R);
          Canvas.UnLock;
          if (R.Right > R.Left) and (R.Bottom > R.Top) then begin
            C[1] := FCoordinate[1] * ClientWidth / (R.Right - R.Left);
            C[3] := FCoordinate[3] * ClientHeight / (R.Bottom - R.Top);
            C[0] := FCoordinate[0] - R.Left / ClientWidth * C[1];
            C[2] := FCoordinate[2] - (ClientHeight - R.Bottom) / ClientHeight * C[3];
            StopAndWait;
            Move(C, FCoordinate, SizeOf(C));
            Invalidate;
          end;
        end;
      pcmMove:
        begin
          Invalidate;
        end;
    end;
    if Assigned(FAggiorna) then FAggiorna(Self);
  end;
  PCMode := pcmNone;
  Cursor := crDefault;
end;

procedure TPianoCartesiano.FMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if FAuto then
    case FPCMode of
      pcmZoom, pcmInvZoom:
        begin
          Canvas.Lock;
          Canvas.DrawFocusRect(RaddrizzaRect(FFocusRect));
          FFocusRect.BottomRight.X := X;
          FFocusRect.BottomRight.Y := Y;
          Canvas.DrawFocusRect(RaddrizzaRect(FFocusRect));
          Canvas.UnLock;
        end;
      pcmMove:
        begin
          FDelta.X := X - FStart.X;
          FDelta.Y := Y - FStart.Y;
          StopAndWait;
          FCoordinate[0] := FStCoord[0] - FDelta.X / ClientWidth * FCoordinate[1];
          FCoordinate[2] := FStCoord[2] + FDelta.Y / ClientHeight * FCoordinate[3];
          Invalidate;
        end;
  end;
  Hint := Format('X: %7.3f - Y: %7.3f', [Delphi2RealX(X), Delphi2RealY(Y)]);
end;

procedure TPianoCartesiano.SetPCMode(Value: TPCMode);
begin
  FPCMode := Value;
end;

constructor TPianoTraiettorie.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFunzioni := TList.Create;
  FMulti := TList.Create;
end;

destructor TPianoTraiettorie.Destroy;
begin
  RemoveAll;
  FFunzioni.Free;
  FMulti.Free;
  inherited Destroy;
end;

procedure TPianoTraiettorie.AddFunzione(Funzione: TFunzione);
begin
  StopAndWait;
  FFunzioni.Add(Funzione);
  Invalidate;
end;

procedure TPianoTraiettorie.AddMulti(Funzione: TMultiFunzione);
begin
  StopAndWait;
  FMulti.Add(Funzione);
  Invalidate;
end;

procedure TPianoTraiettorie.RemoveFunzione(Funzione: TFunzione);
begin
  StopAndWait;
  FFunzioni.Remove(Funzione);
end;

procedure TPianoTraiettorie.RemoveMulti(Funzione: TMultiFunzione);
begin
  StopAndWait;
  FMulti.Remove(Funzione);
end;

procedure TPianoTraiettorie.RemoveAll;
var
  I: Integer;
begin
  StopAndWait;

  for I := 0 to FFunzioni.Count - 1 do TObject(FFunzioni.Items[I]).Free;
  FFunzioni.Clear;

  for I := 0 to FMulti.Count - 1 do TObject(FMulti.Items[I]).Free;
  FMulti.Clear;

  Invalidate;
end;

procedure TPianoTraiettorie.DisegnaThread(Th: TThread);
var
  X1, X2: Integer;
  I, J, N: Integer;
  X, Y: Integer;
  F: TFunzione;
  M: TMultiFunzione;
  P: PExtendedArray0;
  Valido: Boolean;
const
  MAXPERIOD = 129;
begin
  inherited DisegnaThread(Th);

  with Canvas do begin
    GetMem(P, SizeOf(Extended) * MAXPERIOD);

    X1 := FDRect.Left;
    X2 := FDRect.Right;

    try
      Pen.Color := clYellow;
      for I := 0 to FFunzioni.Count - 1 do begin
        F := TFunzione(FFunzioni.Items[I]);
        Valido := False;
        for X := X1 to X2 do begin
          if (Th as TRepaintThread).Terminated then
            raise Exception.Create('Done!');
          Canvas.Lock;
          try
            Y := Real2DelphiY(F.Valuta(Delphi2RealX(X)));
            if Valido then
              LineTo(X, Y)
            else
              MoveTo(X, Y);
            Valido := True;
          except
            Valido := False;
          end;
          Canvas.Unlock;
        end;
      end;

      for I := 0 to FMulti.Count - 1 do begin
        M := TMultiFunzione(FMulti.Items[I]);
        for X := X1 to X2 do begin
          if (Th as TRepaintThread).Terminated then
            raise Exception.Create('Done!');
          Canvas.Lock;
          try
            N := MAXPERIOD;
            M.Valuta(Delphi2RealX(X), N, P);
            if N > 0 then
              if not(IsInfinite(P[0]) or IsNAN(P[0])) then begin
                for J := 0 to N - 1 do begin
                  Y := Real2DelphiY(P[J]);
                  Pixels[X, Y] := clWhite;
                end;
              end;
          except
          end;
          Canvas.Unlock;
        end;
      end;

    except
    end;
  end;
  FreeMem(P);
end;

constructor TPianoIterazione.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIter := 100;
  FSkip := 0;
  FStart := 0.321;
end;

procedure TPianoIterazione.SetStart(Value: Extended);
begin
  if Value <> FStart then begin
    StopAndWait;
    FStart := Value;
    Invalidate;
  end;
end;

procedure TPianoIterazione.SetIter(Value: Integer);
begin
  if Value <> FIter then begin
    StopAndWait;
    FIter := Value;
    Invalidate;
  end;
end;

procedure TPianoIterazione.SetSkip(Value: Integer);
begin
  if Value <> FSkip then begin
    StopAndWait;
    FSkip := Value;
    Invalidate;
  end;
end;

procedure TPianoIterazione.SetFunzione(Value: TFunzione);
begin
  StopAndWait;
  FFunzione.Free;
  FFunzione := Value;
  Invalidate;
end;

procedure TPianoIterazione.DisegnaVCL(Bitmap: TBitmap);
var
  RX: Extended;
  X1, X2: Integer;
  X, Y: Integer;
begin
  inherited DisegnaVCL(Bitmap);

  with Bitmap.Canvas do begin
    X1 := FDRect.Left;
    X2 := FDRect.Right;

    if FFunzione <> nil then
      for X := X1 to X2 do begin
        try
          RX := Delphi2RealX(X);
          Y := Real2DelphiY(FFunzione.Valuta(RX));
          Pixels[X, Y] := clYellow;
        except
        end;
      end;

    Pen.Color := clRed;
    MoveTo(X1, Real2DelphiY(Delphi2RealX(X1)));
    LineTo(X2, Real2DelphiY(Delphi2RealX(X2)));
  end;
end;

procedure TPianoIterazione.DisegnaThread(Th: TThread);
var
  IX, IY: Extended;
  It: Integer;
  I: Integer;
begin
  inherited DisegnaThread(Th);

  if FFunzione <> nil then begin
    IX := FStart;
    It := FIter;
    for I := 1 to FSkip do IX := FFunzione.Valuta(IX);

    with Canvas do begin

      Pen.Color := clBlue;
      try
        IY := 0;
        MoveTo(Real2DelphiX(IX), Real2DelphiY(IY));
        for I := 0 to It do begin
          if (Th as TRepaintThread).Terminated then
            raise Exception.Create('Done!');
          Canvas.Lock;
          IY := FFunzione.Valuta(IX);
          LineTo(Real2DelphiX(IX), Real2DelphiY(IY));
          IX := IY;
          LineTo(Real2DelphiX(IX), Real2DelphiY(IY));
          Canvas.Unlock;
        end;
      except
      end;
    end;
  end;
end;

constructor TPianoIterata.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStart := 0.5;
end;

procedure TPianoIterata.SetStart(Value: Extended);
begin
  if Value <> FStart then begin
    StopAndWait;
    FStart := Value;
    Invalidate;
  end;
end;

procedure TPianoIterata.SetLambda(Value: Extended);
begin
  if Value <> FLambda then begin
    StopAndWait;
    FLambda := Value;
    Invalidate;
  end;
end;

procedure TPianoIterata.SetFunzione(Value: TFunzione);
begin
  StopAndWait;
  FFunzione.Free;
  FFunzione := Value;
  Invalidate;
end;

procedure TPianoIterata.SetMulti(Value: TMultiFunzione);
begin
  StopAndWait;
  FMulti.Free;
  FMulti := Value;
  Invalidate;
end;

procedure TPianoIterata.DisegnaVCL(Bitmap: TBitmap);
var
  RX: Extended;
  X1, X2: Integer;
  X, Y: Integer;
  Valido: Boolean;
begin
  inherited DisegnaVCL(Bitmap);

  with Bitmap.Canvas do begin
    X1 := FDRect.Left;
    X2 := FDRect.Right;
    Pen.Color := clYellow;

    if FFunzione <> nil then begin
      Valido := False;
      for X := X1 to X2 do begin
        try
          RX := Delphi2RealX(X);
          Y := Real2DelphiY(FFunzione.Valuta(RX));
          if Valido then
            LineTo(X, Y)
          else begin
            MoveTo(X, Y);
            Valido := True;
          end;
        except
          Valido := False;
        end;
      end;
    end;

    Pen.Color := clRed;
    MoveTo(X1, Real2DelphiY(Delphi2RealX(X1)));
    LineTo(X2, Real2DelphiY(Delphi2RealX(X2)));
  end;
end;

procedure TPianoIterata.DisegnaThread(Th: TThread);
var
  Y: Extended;
  X1, X2, X: Integer;
  I, N: Integer;
  P: PExtendedArray0;
  Valido: Boolean;
begin
  inherited DisegnaThread(Th);

  with Canvas do begin

    X1 := FDRect.Left;
    X2 := FDRect.Right;

    if (FFunzione <> nil) and (FMulti <> nil) then begin
      try
        GetMem(P, SizeOf(Extended) * 17);
        N := 17;
        Multi.Valuta(FLambda, N, P);
        if (N > 0) and (N < 17) then
          if not(IsInfinite(P[0]) or IsNAN(P[0])) then begin

            Pen.Color := clWhite;
            Valido := False;
            for X := X1 to X2 do begin
              if (Th as TRepaintThread).Terminated then
                raise Exception.Create('Done!');
              Canvas.Lock;
              try
                Y := Delphi2RealX(X);
                for I := 1 to N do Y := FFunzione.Valuta(Y);
                if Valido then
                  LineTo(X, Real2DelphiY(Y))
                else
                  MoveTo(X, Real2DelphiY(Y));
                Valido := True;
              except
                Valido := False;
              end;
              Canvas.UnLock;
            end;

            Pen.Color := clBlue;
            MoveTo(Real2DelphiX(P[0]), Real2DelphiY(P[0]));
            P[N] := P[0];
            for I := 0 to N - 1 do begin
              if (Th as TRepaintThread).Terminated then
                raise Exception.Create('Done!');
              Canvas.Lock;
              LineTo(Real2DelphiX(P[I]), Real2DelphiY(P[I + 1]));
              LineTo(Real2DelphiX(P[I + 1]), Real2DelphiY(P[I + 1]));
              Canvas.Unlock;
            end;
          end;
      except
      end;
      FreeMem(P);
    end;
  end;
end;

constructor TPianoFasi.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCoordinate[0] := -10;
  FCoordinate[1] := 20;
  FInizio := 0;
  FFine := 0;
  FPasso := 0;
end;

destructor TPianoFasi.Destroy;
begin
  StopAndWait;
  if FX <> nil then FX.Free;
  if FY <> nil then FY.Free;
  inherited Destroy;
end;

procedure TPianoFasi.SetX(Value: TFunzione);
begin
  StopAndWait;
  FX := Value;
  Invalidate;
end;

procedure TPianoFasi.SetY(Value: TFunzione);
begin
  StopAndWait;
  FY := Value;
  Invalidate;
end;

procedure TPianoFasi.SetParametri(Inizio, Fine, Passo: Extended);
begin
  StopAndWait;
  FInizio := Inizio;
  FFine := Fine;
  FPasso := Passo;
  Invalidate;
end;

procedure TPianoFasi.DisegnaThread(Th: TThread);
var
  X, Y: Integer;
  T: Extended;
begin
  inherited DisegnaThread(Th);

  if (FX <> nil) and (FY <> nil) then begin
    with Canvas do begin

      try
        T := FInizio;
        X := Real2DelphiX(FX.Valuta(T));
        Y := Real2DelphiY(FY.Valuta(T));
        MoveTo(X, Y);
        T := T + FPasso;
        while T < FFine do begin
          if (Th as TRepaintThread).Terminated then
            raise Exception.Create('Done!');
          Pen.Color := $000000FF + Trunc($FF * (FFine - T) / (FFine - FInizio)) * $100;
          X := Real2DelphiX(FX.Valuta(T));
          Y := Real2DelphiY(FY.Valuta(T));
          Canvas.Lock;
          LineTo(X, Y);
          Canvas.UnLock;
          T := T + FPasso;
        end;
      except
      end;

    end;
  end;
end;

constructor TSizePianoCartesiano.Create(AOwner: TComponent);
var
  NewLeft, NewTop, I: Integer;
const
  Labels: array[0..3] of String = ('MinX:', 'DX:', 'MinY:', 'DY:');
begin
  inherited Create(AOwner);
  Caption := '';
  NewTop := 3;
  NewLeft := 3;
  for I := 0 to 3 do begin
    with TLabel.Create(Self) do begin
      Parent := Self;
      Top := NewTop;
      Left := NewLeft;
      Caption := Labels[I];
    end;
    FMinMaxXY[I] := TFloatSpinEdit.Create(Self);
    with FMinMaxXY[I] do begin
      Increment := 1;
      Parent := Self;
      Top := NewTop;
      Left := NewLeft + 50;
      Tag := I;
      OnChange := OnChangeMinDXY;
      if Odd(I) then begin
        Inc(NewTop, Height);
        NewLeft := 3;
      end else
        Inc(NewLeft, 200)
    end;
  end;
  Height := NewTop + 7;
end;

procedure TSizePianoCartesiano.SetPianoCartesiano(APianoCartesiano: TPianoCartesiano);
var
  I: Integer;
begin
  FPianoCartesiano := APianoCartesiano;
  if Assigned(FPianoCartesiano) then begin
    for I := 0 to 3 do
      FMinMaxXY[I].Value := FPianoCartesiano.MinDXY[I];
    FPianoCartesiano.OnAggiorna := Aggiorna;
  end else
    for I := 0 to 3 do FMinMaxXY[I].Value := 0;
end;

procedure TSizePianoCartesiano.OnChangeMinDXY(Sender: TObject);
var
  E: Extended;
  T: Integer;
begin
  if Assigned(FPianoCartesiano) then begin
    E := TFloatSpinEdit(Sender).Value;
    T := TComponent(Sender).Tag;
    if not(Odd(T) and (E <= 0)) then FPianoCartesiano.MinDXY[T] := E
  end;
end;

procedure TSizePianoCartesiano.Aggiorna(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to 3 do
    FMinMaxXY[I].Value := FPianoCartesiano.MinDXY[I]
end;

procedure Register;
begin
  RegisterComponents('AndSoft Inc.', [TPianoTraiettorie]);
  RegisterComponents('AndSoft Inc.', [TPianoFasi]);
  RegisterComponents('AndSoft Inc.', [TPianoIterazione]);
  RegisterComponents('AndSoft Inc.', [TPianoIterata]);
  RegisterComponents('AndSoft Inc.', [TSizePianoCartesiano]);
end;

end.
